// Airbrush mode

#include <iostream>
#include <vector>

#include "./common.h"

using namespace std;

GLint window_width = 500;
GLint window_height = 500;

struct Point2 {
  Point2(GLint _x, GLint _y) {
    x[0] = _x;
    x[1] = _y;
  }
  operator GLint*() { return x; }
  GLint operator[](int i) const { return x[i]; }
  GLint x[2];
};

struct Color {
  Color(GLfloat _r, GLfloat _g, GLfloat _b) {
    c[0] = _r;
    c[1] = _g;
    c[2] = _b;
  }
  operator GLfloat*() { return c; }
  GLfloat c[3];
};

const Color kRed = Color(1, 0, 0);
const Color kGreen = Color(0, 1, 0);
const Color kBlue = Color(0, 0, 1);

// Returns a random number in the range [a, b]
GLfloat Random(int a, int b) {
  return a + (random() / (double)RAND_MAX) * (b - a);
}

const int kNumDots = 100;
const int kRadius = 10;

vector<Point2> points;
Color color = kRed;

// What unusual behavior will we see?
// A) Dots will change colors dynamically
// B) Dots will move around as we click different places
// C) Dots will change size
void Display() {
  glClear(GL_COLOR_BUFFER_BIT);
  glColor3fv(color);
  glBegin(GL_POINTS);
  for (int i = 0; i < points.size(); ++i) {
    for (int j = 0; j < kNumDots; ++j) {
      glVertex2f(points[i][0] + Random(-kRadius, kRadius),
                 points[i][1] + Random(-kRadius, kRadius));
    }
  }
  glEnd();
  glFlush();
  glutSwapBuffers();
}

void Init() {
  glClearColor(1.0, 1.0, 1.0, 1.0);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, window_width, 0, window_height);
  glMatrixMode(GL_MODELVIEW);
}

void Keyboard(unsigned char key, int x, int y) {
  switch (key) {
    case 'r':
      color = kRed;
      break;
    case 'g':
      color = kGreen;
      break;
    case 'b':
      color = kBlue;
      break;
    case 'q':
      exit(EXIT_SUCCESS);
      break;
  }
}

void Mouse(int button, int state, int x, int y) {
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      points.push_back(Point2(x, window_height-y));
    }
  }

  glutPostRedisplay();
}

void MouseMotion(int x, int y) {
  points.push_back(Point2(x, window_height-y));
  glutPostRedisplay();
}

void Menu(int id) {
  if (id == 1) {
    color = kRed;
  } else if (id == 2) {
    color = kGreen;
  } else if (id == 3) {
    color = kBlue;
  }

  glutPostRedisplay();
}

int main(int argc, char** argv) {
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize(window_width, window_height);
  glutInitWindowPosition(0, 0);
  glutCreateWindow("lecture 3");
  glutDisplayFunc(Display);
  glutKeyboardFunc(Keyboard);
  glutMouseFunc(Mouse);
  glutMotionFunc(MouseMotion);

  Init();

  glutCreateMenu(Menu);
  glutAddMenuEntry("red", 1);
  glutAddMenuEntry("green", 2);
  glutAddMenuEntry("blue", 3);
  glutAttachMenu(GLUT_RIGHT_BUTTON);

  glutMainLoop();
}

