// Airbrush mode with time-sensitive filling

#include <iostream>
#include <vector>

#include "./common.h"

using namespace std;

GLint window_width = 500;
GLint window_height = 500;

struct Color {
  Color() {}
  Color(GLfloat _r, GLfloat _g, GLfloat _b) {
    c[0] = _r;
    c[1] = _g;
    c[2] = _b;
  }
  operator GLfloat*() { return c; }
  GLfloat c[3];
};

struct Point2 {
  Point2(GLint _x, GLint _y, Color _c) {
    x[0] = _x;
    x[1] = _y;
    c = _c;
  }
  operator GLint*() { return x; }
  GLint operator[](int i) const { return x[i]; }
  GLint x[2];
  Color c;
};

const Color kRed = Color(1, 0, 0);
const Color kGreen = Color(0, 1, 0);
const Color kBlue = Color(0, 0, 1);

// Returns a random number in the range [a, b]
GLfloat Random(int a, int b) {
  return a + (random() / (double)RAND_MAX) * (b - a);
}

const int kNumDots = 100;
const int kRadius = 10;

vector<Point2> points;
Color color = kRed;
Point2 mouse_pnt(0, 0, color);

// What unusual behavior will we see?
// A) Dots will change colors dynamically
// B) The last point will continue to fill in after we release the mouse
// C) All points will continue to fill in after we release the mouse
void Display() {
  glClear(GL_COLOR_BUFFER_BIT);
  glBegin(GL_POINTS);
  for (int i = 0; i < points.size(); ++i) {
    glColor3fv(points[i].c);
    glVertex2iv(points[i]);
  }
  glEnd();
  glFlush();
  glutSwapBuffers();
}

void Init() {
  glClearColor(1.0, 1.0, 1.0, 1.0);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, window_width, 0, window_height);
  glMatrixMode(GL_MODELVIEW);
}

void Keyboard(unsigned char key, int x, int y) {
  switch (key) {
    case 'r':
      color = kRed;
      break;
    case 'g':
      color = kGreen;
      break;
    case 'b':
      color = kBlue;
      break;
    case 'q':
      exit(EXIT_SUCCESS);
      break;
  }
}

void Mouse(int button, int state, int x, int y) {
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      y = window_height - y;
      mouse_pnt = Point2(x, y, color);
      for (int j = 0; j < kNumDots; ++j) {
        points.push_back(Point2(
            x + static_cast<int>(Random(-kRadius, kRadius)),
            y + static_cast<int>(Random(-kRadius, kRadius)),
            color));
      }
    }
  }

  glutPostRedisplay();
}

void MouseMotion(int x, int y) {
  y = window_height - y;
  mouse_pnt = Point2(x, y, color);
  for (int j = 0; j < kNumDots; ++j) {
    points.push_back(Point2(
        x + static_cast<int>(Random(-kRadius, kRadius)),
        y + static_cast<int>(Random(-kRadius, kRadius)),
        color));
  }
  glutPostRedisplay();
}

GLfloat last_time = 0;
GLfloat elapsed_time = 0;
void Idle() {
  GLfloat t = glutGet(GLUT_ELAPSED_TIME);
  elapsed_time += t - last_time;
  last_time = t;
  GLfloat num_seconds = t / 1000.0f;
  const int n = num_seconds * 2;
  for (int i = 0; i < n; ++i) {
    points.push_back(Point2(
        mouse_pnt[0] + static_cast<int>(Random(-kRadius, kRadius)),
        mouse_pnt[1] + static_cast<int>(Random(-kRadius, kRadius)),
        mouse_pnt.c));
  }

  glutPostRedisplay();
}

void Menu(int id) {
  if (id == 1) {
    color = kRed;
  } else if (id == 2) {
    color = kGreen;
  } else if (id == 3) {
    color = kBlue;
  }

  glutPostRedisplay();
}

int main(int argc, char** argv) {
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize(window_width, window_height);
  glutInitWindowPosition(0, 0);
  glutCreateWindow("lecture 3");
  glutDisplayFunc(Display);
  glutKeyboardFunc(Keyboard);
  glutMouseFunc(Mouse);
  glutMotionFunc(MouseMotion);
  glutIdleFunc(Idle);

  Init();

  glutCreateMenu(Menu);
  glutAddMenuEntry("red", 1);
  glutAddMenuEntry("green", 2);
  glutAddMenuEntry("blue", 3);
  glutAttachMenu(GLUT_RIGHT_BUTTON);

  glutMainLoop();
}

